#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "astro.h"
#include "inpop.h"
#include "calceph.h"

#define	INPOP_FILE			"/home/kk/astro/inpop17a_TCB_m100_p100_tcg.dat"
#define	INPOP_SJD			2414105.5
#define	INPOP_LJD			2488985.5

// function number
int nFNum;

/*
=================================================
=			INPOP file open					=
=================================================
*/
int init_inpop(void)
{
	double dTmp;
	int nRes = INPOP_FILE_ERR;

	if (calceph_sopen(INPOP_FILE)){
		nRes = INPOP_NO_ERR;
		if (calceph_sgetconstant("TIMESC", &dTmp)){
			if ((int)dTmp == 0){
				nFNum = FNUM_TT_TDB;
			}else{
				nFNum = FNUM_TCG_TCB;
			}
		}
	}

	return nRes;
}

/*
=================================================
=			close INPOP file					=
=================================================
*/
void close_inpop(void)
{
	calceph_sclose();
}


/*
=================================================
=			Get INPOP time data				=
=================================================
*/
int inpop(LONGJD *pJD_tt, double *pdT)
{
	double dPV[6];
	int nRes = INPOP_READ_ERR;

	// heck interval
	if ((pJD_tt->dDate < INPOP_SJD) || (pJD_tt->dDate >= INPOP_LJD)){
		return nRes;
	}

	// Read data
	if (calceph_scompute(pJD_tt->dDate, pJD_tt->dTime, nFNum, 0, dPV)){
		nRes = INPOP_NO_ERR;
		*pdT = dPV[0];
	}

	return nRes;
}


/*
=============================================
=			TT,TCG to TCB					=
=											=
=ent:	LONGJD *pdJDtt = TT JulianDay	=
=		LONGJD *pdJDtcg = TCG JulianDay	=
=exit:	return double TCB-TCG			=
=		LONGJD *pdJDtcb = TCB JulianDay	=
=============================================
*/
double inpop_tcg_tcb(LONGJD *pdJDtt, LONGJD *pdJDtcg, LONGJD *pdJDtcb)
{
	double	dTCB;

	// Get TCB - TCG
	inpop(pdJDtt, &dTCB);

	// Calc. TCB
	pdJDtcb->dDate = pdJDtcg->dDate;
	pdJDtcb->dTime = pdJDtcg->dTime - (dTCB / DAYSEC);
	ljd_normalize(pdJDtcb);

	return dTCB;
}

/*
=============================================
=			TCB to TT,TCG					=
=											=
=ent:	LONGJD *pdJDtcb = TCB JulianDay	=
=exit:	return double TCB-TCG			=
=		LONGJD *pdJDtt = TT JulianDay	=
=		LONGJD *pdJDtcg = TCG JulianDay	=
=============================================
*/
double inpop_tcb_tt(LONGJD *pdJDtcb, LONGJD *pdJDtt, LONGJD *pdJDtcg)
{
	double	dTCB;
	double nOldDay;

	*pdJDtt = *pdJDtcb;
	do{
		nOldDay = pdJDtt->dTime;
		inpop(pdJDtt, &dTCB);
		pdJDtcg->dDate = pdJDtcb->dDate;
		pdJDtcg->dTime = pdJDtcb->dTime + (dTCB / DAYSEC);
		tcg_tt(pdJDtcg, pdJDtt);
	}while(fabs(nOldDay - pdJDtt->dTime) > 1e-15);

	return dTCB;
}
