/*
*********************************************
*											*
*		UTC TAI sample program				*
*											*
*********************************************
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "atime.h"
#include "xfh_tcb.h"
#include "inpop.h"

#define		VERSION		"atime Ver 2017/11/18\n"
#define		REAPSEC_FILE	"dAT.tbl"

LEAPSEC *pdAT;

/*
=============================================
=		display help						=
=============================================
*/
void disp_help(void)
{
	puts(VERSION);
	puts("atime --- PC(UTC) system time");
	puts("atime Year [Month Day Hour Minute Second] [tt] --- Any UTC time");
	puts("atime jd JD-DATA [tt]--- Any JD");
}

/*
=============================================
=		Read leapsec. table for TT		=
=============================================
*/
int read_leapsec_table(char *pFileName)
{
	FILE *fp;
	LEAPSEC *ptbl;

	unsigned long int ndTCount;
	double sec;
	char szDmy[80];

	/*
	-------------------------------------------
			check parameter line
	-------------------------------------------
	*/
	/* Open File */
	fp = fopen(pFileName, "r");
	if(fp == NULL){
		printf("%s cannot open\n", pFileName);
		return -1;
	}
	/* calc. size */
	ndTCount = 0;
	while(!feof(fp)){
		if (fgets(szDmy, sizeof(szDmy), fp) == NULL){
			break;
		}
		ndTCount++;
	}

	/* memory allocate */
	pdAT = (LEAPSEC *)malloc(ndTCount * sizeof(LEAPSEC));
	if (pdAT == NULL){
		fclose(fp);
		puts("Memory allocate error!");
		return -1;
	}

	/*
	-------------------------------------------
			load reap second data
	-------------------------------------------
	*/
	/* Read data */
	ptbl = pdAT;
	fseek(fp, 0, SEEK_SET);
	while(!feof(fp)){
		if (fgets(szDmy, sizeof(szDmy), fp) == NULL){
			break;
		}
		ptbl->dJD = atof(szDmy);
		sec = atof(&szDmy[10]);
		ptbl->dTime = (sec + DTAI_TT) / DAYSEC;
		ptbl++;
	}
	fclose(fp);

	return 0;
}

int init_file(void)
{
	/* inpop init */
	if (init_inpop() != INPOP_NO_ERR){
		printf("Can not INPOP file!\n");
		return -1;
	}

	/* XHF init */
	if (read_hfdata() != XHF_NO_ERR){
		printf("Can not HF Data file!\n");
		return -1;
	}

	/* DAT table read */
	pdAT = NULL;
	if (read_leapsec_table(REAPSEC_FILE) != 0){
		printf("Can not read %s file!\n", REAPSEC_FILE);
		return -1;
	}
	set_dtt_table(pdAT);

	return 0;
}

void close_file(void)
{
	close_inpop();
	/* Memory free */
	free(pdAT);
}


int main(int argc, char *argv[])
{
	time_t timer;
	struct tm *t_st;

	DATETIME dtime = {{0, 1, 1, 0.0}, {0, 0, 0.0}};
	LONGJD dJD_TT;
	LONGJD dJD;
	LONGJD dJD_TCB;
	double dDAT;
	char *pHead;
	char cTemp;
	int nTTMode;
	int i;
	int bJDFlag = 0;
	int bTTflag = 0;

	/* headder */
	char szDT[] = "dT(ET-UT)";
	char szET[] = "ET ";
	char szUT[] = "UT ";

	char szDTT[] = "dTT(TT-UTC)";
	char szTT[] = "TT ";
	char szUTC[] = "UTC";

	char *pszLTT = "TT";
	char *pszSTT = "tt";
	char *pszLJD = "JD";
	char *pszSJD = "jd";

	/* Set arg data */
	if (argc == 1){
		/* get now time */
		time(&timer);
		t_st = gmtime(&timer);
		dtime.date.dYear = t_st->tm_year+1900;
		dtime.date.nMonth = t_st->tm_mon+1;
		dtime.date.nDay = t_st->tm_mday;
		dtime.time.nHour = t_st->tm_hour;
		dtime.time.nMin = t_st->tm_min;
		dtime.time.dSec = t_st->tm_sec;
	}else{
		i = 1;
		if (i != 0){
			/* check help(-h or -?) */
			if (argv[i][0] == '-'){
				cTemp = argv[i][1];
				if ((cTemp == 'h') || (cTemp == 'H') || (cTemp == '?')){
					disp_help();
					return -1;
				}
			}
			/* check JD input */
			if ((strcmp(argv[i], pszLJD) == 0) || (strcmp(argv[i], pszSJD) == 0)){
				bJDFlag = -1;
			}else{
				dtime.date.dYear = atof(argv[i]);
			}
			i++;
		}
		if (i < argc){
			/* check JD input */
			if (bJDFlag != 0){
				dtime.date.dYear = atof(argv[i]);
				if (++i < argc){
					if ((strcmp(argv[i], pszLTT) == 0) || (strcmp(argv[i], pszSTT) == 0)){
						bTTflag = -1;
					}
				}
				i = argc;
			}else{
				if ((strcmp(argv[i], pszLTT) == 0) || (strcmp(argv[i], pszSTT) == 0)){
					bTTflag = -1;
				}else{
					dtime.date.nMonth = atoi(argv[i]);
				}
				i++;
			}
		}
		if (i < argc){
			if ((strcmp(argv[i], pszLTT) == 0) || (strcmp(argv[i], pszSTT) == 0)){
				bTTflag = -1;
			}else{
				dtime.date.nDay = atoi(argv[i]);
			}
			i++;
		}
		if (i < argc){
			if ((strcmp(argv[i], pszLTT) == 0) || (strcmp(argv[i], pszSTT) == 0)){
				bTTflag = -1;
			}else{
				dtime.time.nHour = atoi(argv[i]);
			}
			i++;
		}
		if (i < argc){
			if ((strcmp(argv[i], pszLTT) == 0) || (strcmp(argv[i], pszSTT) == 0)){
				bTTflag = -1;
			}else{
				dtime.time.nMin = atoi(argv[i]);
			}
			i++;
		}
		if (i < argc){
			if ((strcmp(argv[i], pszLTT) == 0) || (strcmp(argv[i], pszSTT) == 0)){
				bTTflag = -1;
			}else{
				dtime.time.dSec = atof(argv[i]);
			}
			i++;
		}
		if (i < argc){
			if ((strcmp(argv[i], pszLTT) == 0) || (strcmp(argv[i], pszSTT) == 0)){
				bTTflag = -1;
			}
		}
	}

	if (init_file() != 0){
		return -1;
	}

	/* check JD input */
	if (bJDFlag != 0){
		jdtoljd(dtime.date.dYear, &dJD);
		if (bTTflag != 0){
			tt_utc(&dJD, &dtime);
		}else{
			juian_dtime_ljd(&dJD, &dtime);
		}
	}else{
		if (bTTflag != 0){
			dtime_julian_ljd(&dtime, &dJD);
			tt_utc(&dJD, &dtime);
		}
	}
	/* display input time */
	nTTMode = 0;
	if (dtime.date.dYear >= 1972.0){
		nTTMode = -1;
	}

	if (nTTMode != 0){
		pHead = szUTC;
	}else{
		pHead = szUT;
	}
	printf("%s %04.0f/%02d/%02d %02d:%02d:%09.6f", pHead, dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);
	printf(" JD = %f\n", dtime_julian(&dtime));

	/* UTC to TT or ET */
	dDAT = utc_tt(&dtime, &dJD_TT);
	juian_dtime_ljd(&dJD_TT, &dtime);
	if (nTTMode != 0){
		pHead = szTT;
	}else{
		pHead = szET;
	}
	printf("%s %04.0f/%02d/%02d %02d:%02d:%09.6f\n", pHead, dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	/* TCG */
	dDAT = tt_tcg(&dJD_TT, &dJD);
	juian_dtime_ljd(&dJD, &dtime);
	printf("TCG %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	/* TCB */
	dDAT = xhf_tcg_tcb(&dJD_TT, &dJD, &dJD_TCB);
	dDAT = inpop_tcg_tcb(&dJD_TT, &dJD, &dJD_TCB);
	juian_dtime_ljd(&dJD_TCB, &dtime);
	printf("XCB %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	dDAT = inpop_tcg_tcb(&dJD_TT, &dJD, &dJD_TCB);
	juian_dtime_ljd(&dJD_TCB, &dtime);
	printf("TCB %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);
	/* TCB xhf */

	/* TDB */
	dDAT = tcb_tdb(&dJD_TCB, &dJD);
	juian_dtime_ljd(&dJD, &dtime);
	printf("TDB %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	printf("\n");

	/* TDB to TCB */
	dDAT = tdb_tcb(&dJD, &dJD_TCB);
	juian_dtime_ljd(&dJD_TCB, &dtime);
	printf("TCB %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	/* TCB to TT */
//	xhf_tcb_tt(&dJD_TCB, &dJD_TT, &dJD);
	inpop_tcb_tt(&dJD_TCB, &dJD_TT, &dJD);
	juian_dtime_ljd(&dJD, &dtime);
	printf("TCG %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);
	juian_dtime_ljd(&dJD_TT, &dtime);
	printf("TT  %04.0f/%02d/%02d %02d:%02d:%09.6f\n", dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	/* TT to UTC */
	tt_utc(&dJD_TT, &dtime);
	if (nTTMode != 0){
		pHead = szUTC;
	}else{
		pHead = szUT;
	}
	printf("%s %04.0f/%02d/%02d %02d:%02d:%09.6f\n", pHead, dtime.date.dYear, dtime.date.nMonth, dtime.date.nDay, dtime.time.nHour, dtime.time.nMin, dtime.time.dSec);

	close_file();

	return 0;
}
